## Copyright (C) 2004 Paul Kienzle <pkienzle@users.sf.net>
## This program is in the public domain

## -*- texinfo -*-
## @deftypefn {Function File} {} zoom
## Compatibility function; does nothing.
## @end deftypefn

function zoom endfunction
